unit uOpcionesSimSEEEdit;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFDEF WINDOWS}
  Windows,
{$ENDIF}
{$IFDEF FPC}
  FileUtil,
{$ENDIF}
  uCosa, uconstantesSimSEE, SysUtils, Dialogs, xmatdefs, utilidades,
  uconversions;

resourcestring
  mesErrorLeyendoArchivoPref =
    'Ocurrio un error leyendo el archivo de preferencias del editor.';
  mesUsaronValoresXDefecto = 'Se usaran los valores por defecto';

const
  configFile = '.\config.ini';

type

  { TSimSEEEditOptions }

  TSimSEEEditOptions = class(TCosa)
  public

  (**************************************************************************)
  (*              A T R I B U T O S   P E R S I S T E N T E S               *)
  (**************************************************************************)

    fechasAutomaticas: boolean;
    fLibPath: string;
    deshabilitarScrollHorizontalEnListados: boolean;
    guardarBackupDeArchivos: boolean;
    maxNBackups: integer;

  (**************************************************************************)

    constructor Create(capa: integer; fechasAutomaticas: boolean;
      deshabilitarScrollHorizontalEnListados: boolean; libPath: string;
      guardarBackupDeArchivos: boolean; maxNBackups: integer);

    //Van public por el warning. No usar, usar getInstance
    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}

    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa: TClaseDeCosa; xVersion: integer = -2): TDataColumnListOfCosa; override;
    class function CreateDataConversionList(): TListDataConversion; override;
    {$ENDIF}

    class function getInstance: TSimSEEEditOptions;
    procedure setLibPath(newLibPath: string);
    procedure guardar;
    procedure Free; override;
    procedure AfterInstantiation;override;

    property libPath: string read fLibPath write setLibPath;

  published
    {$IFDEF MIGRACION_PERSISTENCIA}
    property _fLibPath:                               string  read fLibPath                               write fLibPath;
    property _fechasAutomaticas:                      boolean read fechasAutomaticas                      write fechasAutomaticas;
    property _deshabilitarScrollHorizontalEnListados: boolean read deshabilitarScrollHorizontalEnListados write deshabilitarScrollHorizontalEnListados;
    property _guardarBackupDeArchivos:                boolean read guardarBackupDeArchivos                write guardarBackupDeArchivos;
    property _maxNBackups:                            integer read maxNBackups                            write maxNBackups;
    {$ENDIF}
  end;

implementation

var
  opciones: TSimSEEEditOptions;

constructor TSimSEEEditOptions.Create(capa: integer; fechasAutomaticas: boolean;
  deshabilitarScrollHorizontalEnListados: boolean; libPath: string;
  guardarBackupDeArchivos: boolean; maxNBackups: integer);
begin
  if opciones <> nil then
    raise Exception.Create(
      'TSimSEEEditOptions.Create: Error se esta creando una segunda instancia de TSimSEEEditOptions');

  inherited Create(capa);
  self.fechasAutomaticas := fechasAutomaticas;
  self.deshabilitarScrollHorizontalEnListados := deshabilitarScrollHorizontalEnListados;
  self.libPath := libPath;
  setLibPath(libPath);
  opciones := self;
  self.guardarBackupDeArchivos := guardarBackupDeArchivos;
  Self.maxNBackups := maxNBackups;
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TSimSEEEditOptions.Create_ReadFromText(f: TArchiTexto);
begin
  if opciones <> nil then
    raise Exception.Create(
      'TSimSEEEditOptions.Create_ReadFromText: Error se esta creando una segunda instancia de TSimSEEEditOptions');

  if f.Version < 14 then
  begin
    inherited Create_ReadFromText(f);
    f.IniciarLecturaRetrasada;
    f.rd('fechasAutomaticas', fechasAutomaticas);
    f.rd('libPath', fLibPath);
    f.EjecutarLectura;
    setLibPath(libPath);

    deshabilitarScrollHorizontalEnListados := True;

    guardarBackupDeArchivos := True;
    maxNBackups := 10;
  end
  else if f.Version < 15 then
       begin
         inherited Create_ReadFromText(f);
         f.IniciarLecturaRetrasada;
         f.rd('fechasAutomaticas', fechasAutomaticas);
         f.rd('deshabilitarScrollHorizontalEnListados', deshabilitarScrollHorizontalEnListados);
         f.rd('libPath', fLibPath);
         f.EjecutarLectura;
         setLibPath(libPath);
         guardarBackupDeArchivos := True;
         maxNBackups := 10;
       end
       else
       begin
         inherited Create_ReadFromText(f);
         f.IniciarLecturaRetrasada;
         f.rd('fechasAutomaticas', fechasAutomaticas);
         f.rd('deshabilitarScrollHorizontalEnListados',deshabilitarScrollHorizontalEnListados);
         f.rd('libPath', fLibPath);
         setLibPath(libPath);
         f.rd('guardarBackupDeArchivos', guardarBackupDeArchivos);
         f.rd('maxNBackups', maxNBackups);
         f.EjecutarLectura;
       end;
  opciones := self;
end;
{$ENDIF}

{$IFNDEF MIGRACION_PERSISTENCIA}
procedure TSimSEEEditOptions.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('fechasAutomaticas', fechasAutomaticas);
  f.wr('deshabilitarScrollHorizontalEnListados', deshabilitarScrollHorizontalEnListados);
  f.wr('libPath', quitarRaiz_(fLibPath));
  f.wr('guardarBackupDeArchivos', guardarBackupDeArchivos);
  f.wr('maxNBackups', maxNBackups);
end;
{$ENDIF}

{$IFDEF MIGRACION_PERSISTENCIA}
class function TSimSEEEditOptions.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: integer): TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa, xVersion);
  Result.AddStringColumn ('libPath',                                '',   0,                             VERSION_MIGRACION_PERSISTENCIA);
  Result.AddStringColumn ('flibPath',                               '',   VERSION_MIGRACION_PERSISTENCIA                               );
  Result.AddBooleanColumn('fechasAutomaticas',                      false                                                              );
  Result.AddBooleanColumn('deshabilitarScrollHorizontalEnListados', true, 15                                                           );
  Result.AddBooleanColumn('guardarBackupDeArchivos',                true, 15                                                           );
  Result.AddIntegerColumn('maxNBackups',                            10,   15                                                           );
end;

class function TSimSEEEditOptions.CreateDataConversionList: TListDataConversion;
var
  i: Integer;
  dc: TDataConversion;
  ColumnsInput: TDAofString;
begin
  Result:=inherited CreateDataConversionList();
  Result.Add(VERSION_MIGRACION_PERSISTENCIA,
             ['libPath'],
             ['fLibPath'],
             Result.ConversionNAMECHANGE);
end;
{$ENDIF}

class function TSimSEEEditOptions.getInstance: TSimSEEEditOptions;
var
  f: TArchiTexto;
  {$IFDEF MIGRACION_PERSISTENCIA}
  SalaDeJuegoConnectionHandler: TSalaDeJuegoConnection;
  {$ENDIF}
begin
  if opciones = nil then
  begin
    if FileExists(configFile) then
    begin
      try
        {$IFDEF MIGRACION_PERSISTENCIA}
        DATA_STORE.Clear();
        SalaDeJuegoConnectionHandler:=TSalaDeJuegoConnection.Create(configFile);
        SalaDeJuegoConnectionHandler.ReadDataStoreFromTextFile('opciones');
        DATA_STORE.CreateClassInstance('dt_TSimSEEEditOptions', 1001, TCosa(opciones));
        {$ELSE}
        f := nil;
        f := TArchiTexto.CreateForRead(getDir_Bin + configFile, True);
        f.rd('opciones', TCosa(opciones));
        f.Free;
        {$ENDIF}
      except
        ShowMessage(mesErrorLeyendoArchivoPref + mesUsaronValoresXDefecto);
        if f <> nil then
          f.Free;
        DeleteFile(configFile);
        opciones := TSimSEEEditOptions.Create(0, True, True,
          uconstantesSimSEE.getDir_Lib, True, 10);
      end;

    end
    else
    begin
      opciones := TSimSEEEditOptions.Create(0, True, False,
        uconstantesSimSEE.getDir_Lib, True, 10);
    end;
  end;

  Result := opciones;
end;

procedure TSimSEEEditOptions.setLibPath(newLibPath: string);
begin
  //Si no es una url de red le pongo la letra de unidad
  if (pos('\\', newLibPath) <> 1) and (pos(':\', newLibPath) = 0) then
    fLibPath := ExtractFileDrive(getDir_Bin) + newLibPath
  else
    fLibPath := newLibPath;
  if (Length(fLibPath)= 0) or (fLibPath[Length(fLibPath)] <> barraDir) then
    fLibPath := fLibPath + barraDir;

  if not DirectoryExists(fLibPath) then
    if not ForceDirectories(fLibPath)  then
      raise Exception.Create(
        'TSimSEEEditOptions.setLibPath: No se puede acceder al directorio ' + newLibPath);
end;

procedure TSimSEEEditOptions.guardar;
var
  f: TArchiTexto;
begin
  f := nil;
  try
    ucosa.procMsgValorPorDefecto := nil;
    f := TArchiTexto.CreateForWrite(getDir_Bin + configFile, False, 0);
    f.wr('opciones', opciones);
  finally
    if f <> nil then
      f.Free;
  end;
end;

procedure TSimSEEEditOptions.Free;
begin
  opciones := nil;
  self.Free;
end;

procedure TSimSEEEditOptions.AfterInstantiation;
begin
  setLibPath(libPath);
  opciones := self;
end;

end.
