unit uscangrafico;

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls,
  Graphics, Dialogs, ExtCtrls,
  StdCtrls,
  httpsend, synacode,
  Menus, ComCtrls, Grids, Math, xmatdefs, matreal,
  ute_sge_scan;

(***
http://previsaonumerica.cptec.inpe.br/golMapWeb/ProcessFigura?id=2222&hr=1&mapext=-59.29859999999999+-36.55080000000001+-52.944466666666656+-28.608133333333335&rodada=00
***)




const
  LONG1 = -60.50073333333333;
  LAT2 = -36.95866666666667;
  LONG2 = -51.914066666666656;
  LAT1 = -26.22533333333333;

  FactorDePerdida_V = 0.9; // para ajustar

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    btBorrarBorde: TButton;
    Button2: TButton;
    Button3: TButton;
    btCalcVmedias: TButton;
    btPlotParques: TButton;
    Button4: TButton;
    Button5: TButton;
    btSGE_UTE: TButton;
    btGenGPF: TButton;
    eBaseArchi: TEdit;
    eRodada: TEdit;
    Image1: TImage;
    Label1: TLabel;
    Label2: TLabel;
    Memo1: TMemo;
    Shape1: TShape;
    StatusBar1: TStatusBar;
    sgResultados: TStringGrid;
    procedure btBorrarBordeClick(Sender: TObject);
    procedure btGenGPFClick(Sender: TObject);
    procedure btPlotParquesClick(Sender: TObject);
    procedure btSGE_UTEClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure btCalcVmediasClick(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: integer);
    procedure Image1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: integer);
  private
    { private declarations }
  public
    { public declarations }
    patron_ymin, patron_ymax, patron_cntNegros: array of integer;


    procedure inicializar;

    procedure loadPatrones;
    function distancia(ymin, ymax, cntNegros: array of integer;
      alfa, beta, alfa0, beta0: double): double;

    function CalcVMedia(top, left: integer): double;
  end;




  TParqueEolico = class
    top, left: integer;
    Activo: integer; // -1 Parece que no se hace, 0 En construcción, 1 En Operación.

    CentralGeneradora: string;
    AgenteGenerador: string;
    EstadoActual: string;
    Departamento: string;
    Localidad: string;
    FechaEntradaEnServicio: TDateTime;
    Tension_kV: string;
    Potencia_Autorizada_MW: double;
    Latitud: double;
    Longitud: double;
    Fabricante: string;
    Modelo: string;
    PotenciaAerogenerador_MW: double;
    CantidadAerogeneradores: integer;

    constructor Create(Activo: integer; CentralGeneradora: string;
      AgenteGenerador: string; EstadoActual: string; Departamento: string;
      Localidad: string; FechaEntradaEnServicio: string; // En formato de Fecha Local
      Tension_kV: string; Potencia_Autorizada_MW: double; Latitud: double;
      Longitud: double; Fabricante: string; Modelo: string;
      PotenciaAerogenerador_MW: double; CantidadAerogeneradores: integer);

    procedure CalcTopLeft;
  end;


var
  Form1: TForm1;
  c: TCanvas;
  nFilas, nCols: integer;
  Parques: TList;

implementation

{$R *.lfm}



const
  Velocidades: array[0..13] of double =
    (0, 5, 10, 15, 25, 30, 35, 40, 45, 50, 60, 65, 70, 75);

  TOP_Banner = 100;


var
  ColorVels: array[0..12] of TColor;
  ColorVels_R: array[0..12] of integer;
  ColorVels_G: array[0..12] of integer;
  ColorVels_B: array[0..12] of integer;



constructor TParqueEolico.Create(Activo: integer; CentralGeneradora: string;
  AgenteGenerador: string; EstadoActual: string; Departamento: string;
  Localidad: string; FechaEntradaEnServicio: string; Tension_kV: string;
  Potencia_Autorizada_MW: double; Latitud: double; Longitud: double;
  Fabricante: string; Modelo: string; PotenciaAerogenerador_MW: double;
  CantidadAerogeneradores: integer);
begin
  Self.Activo := Activo;

  Self.CentralGeneradora := CentralGeneradora;
  Self.AgenteGenerador := AgenteGenerador;
  Self.EstadoActual := EstadoActual;
  Self.Departamento := Departamento;
  Self.Localidad := Localidad;
  if (FechaEntradaEnServicio <> '') then
    Self.FechaEntradaEnServicio := StrToDateTime(FechaEntradaEnServicio)
  else
    Self.FechaEntradaEnServicio := StrToDateTime('1/1/3000');

  Self.Tension_kV := Tension_kV;
  Self.Potencia_Autorizada_MW := Potencia_Autorizada_MW;
  Self.Latitud := Latitud;
  Self.Longitud := Longitud;
  Self.Fabricante := Fabricante;
  Self.Modelo := Modelo;
  Self.PotenciaAerogenerador_MW := PotenciaAerogenerador_MW;
  Self.CantidadAerogeneradores := CantidadAerogeneradores;
end;

procedure TParqueEolico.CalcTopLeft;
begin
  Left := trunc((Longitud - Long1) / (Long2 - Long1) * nCols + 0.5);
  Top := trunc((Latitud - Lat1) / (Lat2 - Lat1) * (nFilas - TOP_Banner) + TOP_Banner + 0.5);
end;


procedure Calc_ColorVels_RGB;
var
  k: integer;
  aCol: TColor;
  rgb: longint;
begin
  for k := 0 to 12 do
  begin
    rgb := ColorToRGB(ColorVels[k]);
    ColorVels_R[k] := red(rgb);
    ColorVels_G[k] := green(rgb);
    ColorVels_B[k] := blue(rgb);
  end;

end;

function Velocidad(cl: TColor): double;
var
  r, g, b: integer;
  rgb: integer;
  res: double;
  d, dSel: double;
  k, kSel: integer;
begin
  if (cl = -1) or (cl = clWhite) or (cl = clBlack) then
  begin
    Result := -1;
    exit;
  end;
  rgb := ColorToRGB(cl);
  r := red(rgb);
  g := green(rgb);
  b := blue(rgb);

  kSel := -1;
  dSel := 1;
  for k := 0 to 12 do
  begin
    d := sqrt(sqr(r - ColorVels_R[k]) + sqr(G - ColorVels_G[k]) +
      sqr(B - ColorVels_B[k])) / (sqrt(3.0) * 255);
    if d < dSel then
    begin
      kSel := k;
      dSel := d;
    end;
  end;


  if dSel < 10 then
    Result := (Velocidades[kSel] + Velocidades[kSel + 1]) / 2.0
  else
    Result := -1;
end;



function intensidad(cl: TColor): double;
var
  r, g, b: integer;
  rgb: integer;
  res: double;
begin
  if cl = -1 then
  begin
    Result := 1;
    exit;
  end;
  rgb := ColorToRGB(cl);
  r := red(rgb);
  g := green(rgb);
  b := blue(rgb);
  res := (r + g + b) / 255.0 / 3.0;
  //  if res < 0.5 then writeln( res );
  Result := res;
end;




function TForm1.CalcVMedia(Top, Left: integer): double;
var
  cnt: integer;
  sum: double;
  pixcol: TColor;
  avel: double;
  k1, j1, k2, j2: integer;
  k, j: integer;
begin
  k1 := TOp - 25;
  j1 := Left - 25;
  k2 := Top + 25;
  j2 := Left + 25;


  cnt := 0;
  sum := 0;
  for k := k1 to k2 do
  begin
    for j := j1 to j2 do
    begin
      pixcol := c.Pixels[j, k];
      avel := velocidad(pixcol);
      if avel >= 0 then
      begin
        Inc(cnt);
        sum := sum + avel;
      end;
    end;
  end;
  Result := sum / cnt;
end;


procedure TForm1.loadPatrones;
var
  f: textfile;
  k, k_: integer;
begin
  setlength(patron_ymin, 500);
  setlength(patron_ymax, 500);
  setlength(patron_cntNegros, 500);
  assignfile(f, 'yminmaxcnt.xlt');
  reset(f);
  readln(f); // 'fila'#9'ymin'#9'ymax'#9'cnt');
  for k := 0 to 500 - 1 do
    readln(f, k_, patron_ymin[k], patron_ymax[k], patron_cntNegros[k]);
  closefile(f);
end;


function TForm1.distancia(ymin, ymax, cntNegros: array of integer;
  alfa, beta, alfa0, beta0: double): double;
var
  res: double;
  k, kp: integer;
  cnt_procesados: integer;
  yminp, ymaxp: double;
begin
  res := 0;
  cnt_procesados := 0;
  for k := 0 to high(ymin) do
  begin
    if (cntNegros[k] > 0) and (ymax[k] > ymin[k]) then
    begin
      kp := trunc(k * alfa + alfa0 + 0.5);
      if (kp >= 0) and (kp < 500) and (patron_cntNegros[kp] > 0) and
        (patron_ymax[kp] > patron_ymin[kp]) then
      begin
        yminp := ymin[k] * beta + beta0;
        ymaxp := ymax[k] * beta + beta0;
        res := res + sqr(yminp - patron_ymin[kp]) + sqr(ymaxp - patron_ymax[kp]);
        Inc(cnt_procesados);
      end;
    end;
  end;
  if cnt_procesados > 50 then
    Result := res / (cnt_procesados - 45)
  else
    Result := 1e32;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  r, g, b: integer;
  k, j: integer;
  cl: TColor;
  rgb: integer;
  intensidad: double;

begin
  for k := 0 to nFilas - 1 do
  begin
    for j := 0 to nCols - 1 do
    begin
      cl := c.Pixels[j, k];
      rgb := ColorToRGB(cl);
      r := red(rgb);
      g := green(rgb);
      b := blue(rgb);
      intensidad := (r + g + b) / 255.0 / 3.0;
      if (intensidad < 0.1) then
        c.Pixels[j, k] := clRed;
    end;
  end;
end;



function Scan_FrecRayo(c: TCanvas; kOrg, jOrg: integer; alfa: double;
  // angulo anti-horario desde la horizontal
  largo: integer // pixeles
  ): double;
var
  h: integer;
  dw, dh: integer;
  flg_iniciado: boolean;
  cnt_inversiones: integer;
  ZonaBlanca: boolean;
  fcos, fsin, ff: double;
  aIntensidad: double;
  aCol: TCOlor;
  k, j: integer;
  sx: integer;

  procedure procesar_Acol;
  begin
    aIntensidad := intensidad(acol);
    if aIntensidad > 0 then
    begin
      if flg_iniciado then
      begin
        if ZonaBlanca then
        begin
          if aIntensidad < 0.9 then
          begin
            Inc(cnt_inversiones);
            ZonaBlanca := False;
          end;
        end
        else
        begin
          if aIntensidad > 0.9 then
          begin
            Inc(cnt_inversiones);
            ZonaBlanca := True;
          end;
        end;
      end
      else
      begin
        ZonaBlanca := aIntensidad > 0.9;
        flg_Iniciado := True;
      end;
    end;
  end;

begin

  fcos := cos(pi * alfa / 180.0);
  fsin := sin(pi * alfa / 180.0);

  dw := trunc(largo * fcos + 0.5);
  dh := trunc(largo * fsin + 0.5);
  flg_iniciado := False;
  cnt_inversiones := 0;
  ZonaBlanca := True;

  if abs(dw) > abs(dh) then
  begin
    sx := sign(dw);
    ff := dh / dw * sx;
    for h := 0 to abs(dw) - 1 do
    begin
      k := kOrg + h;
      j := trunc(jOrg + h * ff + 0.5);
      acol := c.Pixels[j, k];
      procesar_ACol;
    end;
  end
  else
  begin
    sx := sign(dh);
    ff := dw / dh * sx;
    for h := 0 to abs(dh) - 1 do
    begin
      k := trunc(kOrg + h * ff + 0.5);
      j := jOrg + h * sx;
      acol := c.Pixels[j, k];
      procesar_ACol;
    end;
  end;
  Result := cnt_inversiones;
end;


procedure DrawRayo(c: TCanvas; kOrg, jOrg: integer; alfa: double;
  // angulo anti-horario desde la horizontal
  largo: integer; // pixeles
  color: TColor);
var
  h: integer;
  dw, dh: integer;
  fcos, fsin, ff: double;
  k, j: integer;
  sx: integer;
begin
  fcos := cos(pi * alfa / 180.0);
  fsin := sin(pi * alfa / 180.0);

  dw := trunc(largo * fcos + 0.5);
  dh := trunc(largo * fsin + 0.5);
  if abs(dw) > abs(dh) then
  begin
    sx := sign(dw);
    ff := dh / dw * sx;
    for h := 0 to abs(dw) - 1 do
    begin
      k := kOrg + h * sx;
      j := trunc(jOrg + h * ff + 0.5);
      c.Pixels[j, k] := Color;
    end;
  end
  else
  begin
    sx := sign(dh);
    ff := dw / dh * sx;
    for h := 0 to abs(dh) - 1 do
    begin
      k := trunc(kOrg + h * ff + 0.5);
      j := jOrg + h * sx;
      c.Pixels[j, k] := Color;
    end;
  end;
end;



procedure TForm1.Inicializar;
var
  alfa: double;
  k: integer;
  radio: double;
  wk, wj: integer;
  delta_k: double;
  ap: TParqueEolico;

begin
  c := Image1.Picture.Bitmap.Canvas;

  nFilas := Image1.Height;
  nCols := Image1.Width;

  alfa := pi / 2.0 / 13;
  radio := 70;
  for k := 0 to 12 do
  begin
    delta_k := 0.5;
    alfa := pi / 2 - (pi / 2.0 / 13.0) * (k + delta_k);
    wj := trunc(nCols - radio * cos(alfa)) + 1;
    wk := trunc(radio * sin(alfa)) - 1 + TOP_Banner;

    colorVels[k] := c.Pixels[wj, wk];
    c.Pixels[wj, wk] := clWhite;
    c.Pixels[wj - 1, wk] := clWhite;
    c.Pixels[wj, wk - 1] := clWhite;
    c.Pixels[wj + 1, wk] := clWhite;
    c.Pixels[wj, wk + 1] := clWhite;
  end;
  Calc_ColorVels_RGB;

  for k := 0 to Parques.Count - 1 do
  begin
    ap := Parques[k];
    ap.CalcTopLeft;
  end;

end;

procedure TForm1.Button2Click(Sender: TObject);
var
  r, g, b: integer;
  k, j: integer;
  cl: TColor;
  dw, dh: integer;
  newLeft, newTop: integer;
  jj: integer;
  cnt: integer;
  pixcol: TCOlor;

  sal: Text;
  sum, avel: double;

begin

  Shape1.Left := nCols - Shape1.Width * 2;
  Shape1.Top := nFilas - Shape1.Height * 2;

  dw := 4;
  dh := 4;


 (**
  assignfile( sal, 'pixs.xlt' );
  rewrite( sal );
  for k:= 1 to 200 do
  begin
   for j:= 1 to 200 do
   begin
      pixcol:= c.Pixels[ j,  k ];
      write( sal, #9, intensidad( pixcol ) );
      c.Pixels[ j,  k ]:= clBlue;
      application.ProcessMessages;
   end;
   writelN( sal );
  end;
  closefile( sal );

     **)


  randomize;
  randomize;
  randomize;

  shape1.Pen.Color := clYellow;

  for jj := 1 to 10000 do
  begin
    newLeft := max(10, Shape1.Left - random(dw) + 1);
    newTop := max(10, Shape1.Top - random(dh) + 1);
    cnt := 0;

    for j := -dw to 1 do
      for k := -dh to 1 do
      begin
        pixcol := c.Pixels[newLeft + j, newTop + k];
        if intensidad(pixcol) < 0.1 then
          Inc(cnt);
      end;


    if (cnt = 0) then
    begin
      Shape1.Left := newLeft;
      Shape1.Top := newTop;
      shape1.Brush.Color := clGreen;
      application.ProcessMessages;
    end
    else
    begin
      shape1.Brush.Color := clRed;
      application.ProcessMessages;
    end;

    //  if (jj mod 10 )=  0 then sleep(1);
  end;

end;

procedure TForm1.Button3Click(Sender: TObject);
var
  c: TCanvas;
  nFilas, nCols: integer;
  kAlfa: integer;
  kOrg, jOrg: integer;
  radio: integer;
  kMenorFrec, kMayorFrec: integer;
  MenorFrec, MayorFrec: double;
  aFrec: double;
  k: integer;
  alfa: double;

begin

  c := Image1.Picture.Bitmap.Canvas;

  nFilas := Image1.Height;
  nCols := Image1.Width;

  kOrg := 250;
  jOrg := 250;
  radio := 100;

  kMenorFrec := 0;
  kMayorFrec := 0;
  MenorFrec := Scan_FrecRayo(c, kOrg, jOrg, 0, radio);
  MayorFrec := MenorFrec;

  for k := 0 to 360 do
  begin
    alfa := k * 1;
    aFrec := Scan_FrecRayo(c, kOrg, jOrg, alfa, radio);
    if aFrec <= MenorFrec then
    begin
      kMenorFrec := k;
      MenorFrec := aFrec;
    end
    else if aFrec >= MayorFrec then
    begin
      kMayorFrec := k;
      MayorFrec := aFrec;
    end;
  end;

  DrawRayo(c, kOrg, jOrg, kMenorFrec * 10, radio, clRed);
  DrawRayo(c, kOrg, jOrg, kMayorFrec * 10, radio, clGreen);
end;

procedure TForm1.btCalcVmediasClick(Sender: TObject);
begin

end;

function pad( k: integer; ndigitos: integer = 2 ): string;
var
  s: string;
begin
  s:= IntToStr( k );
  while length( s ) < ndigitos do s:= '0'+s;
  result:= s;
end;

procedure TForm1.Button4Click(Sender: TObject);
var
   http: THttpSend;
   kHora: integer;
   url: string;
   archi: string;
   fs: longint;
 begin
   randomize;

   for kHora:= 1 to 13 do
   begin
     archi:= 'c:\basura\foto'+eBaseArchi.Text+eRodada.Text+'_'+pad( kHora )+'.png';
     if fileexists( archi ) and ( FileSize( archi ) > ( 30*1024 ) ) then
     begin
       sgResultados.Cells[0, kHora-1]:= IntToStr( kHora );
       sgResultados.Cells[1, kHora-1]:= IntToStr( FileSize( archi ) );
     end
     else
     begin
       application.ProcessMessages;
       http:= THTTPSend.Create;
       url:= 'http://previsaonumerica.cptec.inpe.br/golMapWeb/ProcessSalvarPNG?id=2222&hr='+IntToStr( kHora )+'&mapext=-60.50073333333333%20-36.95866666666667%20-51.914066666666656%20-26.22533333333333&rodada='+eRodada.Text;
       if http.httpmethod( 'GET', url ) then
       begin
         sleep(1);
         sgResultados.Cells[0, kHora-1]:= IntToStr( kHora );
         sgResultados.Cells[1, kHora-1]:= IntToStr( http.Document.Size);
         http.Document.SaveToFile( archi );
       end;
       http.Free;
       sleep(3+random(2));
     end;
   end;
end;

procedure TForm1.Button5Click(Sender: TObject);
var
   anio, mes, dia: word;
   hora, minuto, segundo, miliSegundos: word;
   dtAhora: TDateTime;
   baseArchi: string;
   sRodada: string;
   url: string;
   http: THTTPSend;

begin

     if eRodada.text = '' then
     begin
       if ( hora > 6 ) and (hora < 18) then
         sRodada:= '00'
       else
         sRodada:= '12';
       eRodada.text:= sRodada;
     end;

     if eBaseArchi.Text = '' then
     begin
       dtAhora:= now;
       decodedate( dtAhora, anio, mes, dia );
       DecodeTime( dtAhora, hora, minuto, segundo, miliSegundos );
       BaseArchi:= pad( anio,4 ) +pad( mes) +pad( dia );
       eBaseArchi.Text:= BaseArchi;
     end;

     url:= 'http://previsaonumerica.cptec.inpe.br/golMapWeb/ProcessSalvarPNG?id=2222&hr='+IntToStr( 1 )+'&mapext=-60.50073333333333%20-36.95866666666667%20-51.914066666666656%20-26.22533333333333&rodada='+eRodada.Text;

     writeln( url );

     http:= THTTPSend.Create;
     if http.httpmethod( 'GET', url ) then
        http.Document.SaveToFile( 'c:\basura\foto'+eBaseArchi.Text+eRodada.Text+'_'+pad(1)+'.png' );
     http.Free;

     Image1.Picture.LoadFromFile('c:\basura\foto'+eBaseArchi.Text +eRodada.Text+'_'+pad(1) + '.png');


end;


procedure TForm1.FormCreate(Sender: TObject);
var
  ap: TParqueEolico;
begin
  Parques := TList.Create;


  ap := TParqueEolico.Create(-1, 'LIBERTADOR I',
    'CENTRAL DE GENERACIÓN EÓLICA LIBERTADOR I S.A.', 'Convenio suscripto',
    'LAVALLEJA', 'LAVALLEJA - MALDONADO', '', '63', 50.00, -34.478127,
    -55.138639, 'V87 Impsa Vensys ful converter', '', 0, 0);
  Parques.add(ap);

  ap := TParqueEolico.Create(-1, 'LIBERTADOR II',
    'CENTRAL DE GENERACIÓN EÓLICA LIBERTADOR I S.A.', 'Convenio suscripto',
    'LAVALLEJA', 'LAVALLEJA Y MALDONADO', '', '63', 7.50, -34.47791,
    -55.135116, 'V87 Impsa Vensys ful converter', '', 0, 0);
  Parques.add(ap);

  ap := TParqueEolico.Create(-1, 'LIBERTADOR III',
    'CENTRAL DE GENERACIÓN EÓLICA LIBERTADOR I S.A.', 'Convenio suscripto',
    'MALDONADO', 'MALDONADO', '', '63', 7.50, -34.46791, -55.125116,
    'V87 Impsa Vensys full converter', '', 0, 0);
  Parques.add(ap);

  ap := TParqueEolico.Create(-1, 'LIBERTADOR IV',
    'CENTRAL DE GENERACIÓN EÓLICA ARTIGAS S.A.', 'Convenio suscripto',
    'LAVALLEJA', 'Minas', '', '150', 50.00, -34.422323, -54.980816,
    'V87 Impsa Vensys ful converter', '', 0, 0);
  Parques.add(ap);

  ap := TParqueEolico.Create(0, 'MELOWIND', 'ESTRELLADA S.A.',
    'Convenio suscripto', 'CERRO LARGO', 'Arbolito', '', '150', 50.00,
    -32.582846, -54.230207, 'Nordex', 'N100', 0, 0);
  Parques.add(ap);

  ap := TParqueEolico.Create(-1, 'MOLINO DE ROSAS 50 MW', 'MOLINO DE ROSAS S.A.',
    'Convenio suscripto', 'MALDONADO', 'Pan de Azúcar', '', '150',
    50.00, -34.712553, -55.219573, '', '', 0, 0);
  Parques.add(ap);

  ap := TParqueEolico.Create(0, 'PARQUE CERRO GRANDE', 'LADANER S.A.',
    'Convenio suscripto', 'CERRO LARGO', '', '', '150', 50.00,
    -32.242580, -53.907256, '', '', 0, 0);
  Parques.add(ap);

  ap := TParqueEolico.Create(0, 'PARQUE EÓLICO CARAPÉ I', 'FINGANO S.A.',
    'Convenio suscripto', 'MALDONADO', 'Cerro Catedral', '', '150',
    50.00, -34.395021, -54.695066, 'Vestas', 'V112 - 3MW', 3, 17);
  Parques.add(ap);

  ap := TParqueEolico.Create(0, 'PARQUE EÓLICO KIYÚ', 'COBRA INGENIERÍA URUGUAY S.A.',
    'Convenio suscripto', 'SAN JOSÉ', 'Kiyú', '', '150', 48.60, -34.669759,
    -56.8018, 'Vestas', 'V90 - 3MW', 3, 16);
  Parques.add(ap);

  ap := TParqueEolico.Create(1, 'PARQUE EÓLICO ROSARIO', 'TOGELY COMPANY S.A.',
    'Convenio suscripto', 'COLONIA', 'Rosario', '', '63', 9.00,
    -34.22651, -57.336608, '', '', 0, 0);
  Parques.add(ap);

  ap := TParqueEolico.Create(0, 'PERALTA 1 GCEE', 'AGUA LEGUAS S.A.',
    'Convenio suscripto', 'TACUAREMBÓ', 'Cuchilla Peralta', '', '150',
    50.00, -32.582638, -56.394636, 'Enercon', 'E92', 2.35, 25);
  Parques.add(ap);

  ap := TParqueEolico.Create(0, 'PERALTA 2 GCEE', 'AGUA LEGUAS S.A.',
    'Convenio suscripto', 'TACUAREMBÓ', 'Cuchilla Peralta', '', '150',
    50.00, -32.600400, -56.419570, 'Enercon', 'E92', 2.35, 25);
  Parques.add(ap);

  ap := TParqueEolico.Create(0, 'TALAS DEL MACIEL I', 'ASTIDEY S.A.',
    'Convenio suscripto', 'FLORES', 'Talas del Maciel', '', '150',
    50.00, -33.634783, -56.721357, 'Gamesa', 'Gamesa G97', 0, 0);
  Parques.add(ap);

  ap := TParqueEolico.Create(0, 'TALAS DEL MACIEL II', 'CADONAL S.A.',
    'Convenio suscripto', 'FLORES', 'Talas del Maciel', '', '150',
    50.00, -33.605045, -56.711654, 'Gamesa ', 'G97', 0, 0);
  Parques.add(ap);

  ap := TParqueEolico.Create(1,
    'CARACOLES 1 del PARQUE EÓLICO ING. EMANUELE CAMBILARGIU',
    'UTE', 'En servicio', 'MALDONADO', 'Sierra de los Caracoles',
    '10/02/2009', '31.5', 10.00, -34.626901, -54.957502, 'Vestas', 'V80', 2, 5);
  Parques.add(ap);

  ap := TParqueEolico.Create(1,
    'CARACOLES 2 del PARQUE EÓLICO ING. EMANUELE CAMBILARGIU',
    'UTE', 'En servicio', 'MALDONADO', 'Sierra de los Caracoles',
    '23/06/2010', '31.5', 10.00, -34.648089, -54.966066, 'Vestas', 'V80', 2, 5);
  Parques.add(ap);

  ap := TParqueEolico.Create(1, 'PARQUE DE VACACIONES UTE-ANTEL',
    'UTE-ANTEL', 'En servicio', 'LAVALLEJA', 'Minas', '01/04/2010',
    '6.4', 0.15, -34.425003, -55.183542, '', '', 0, 0);
  Parques.add(ap);

  ap := TParqueEolico.Create(1, 'AGROLAND', 'AGROLAND S.A.',
    'En servicio (Habilitación Final)', 'MALDONADO', 'Paraje Garzón',
    '30/04/2008', '15', 0.25, -34.634532, -54.612077, 'Bonus', '150', 0, 0);
  Parques.add(ap);

  ap := TParqueEolico.Create(1, 'CUCHILLA DEL PERALTA I', 'PALMATIR S.A.',
    'En servicio (Habilitación Final)', 'TACUAREMBÓ', 'Cuchilla Peralta',
    '13/05/2014', '150', 50.00, -32.570107, -56.443977, 'Gamesa', 'Gamesa G90', 2, 25);
  Parques.add(ap);

  ap := TParqueEolico.Create(1, 'LUZ DE MAR', 'LUZ DE MAR S.A.',
    'En servicio (Habilitación Final)', 'FLORIDA', 'PARAJE PINTADO',
    '27/08/2014', '31.5', 18.00, -33.876277, -56.350492, 'Vestas', 'V112 - 3 MW', 3, 6);
  Parques.add(ap);

  ap := TParqueEolico.Create(1, 'LUZ DE RÍO', 'LUZ DE RÍO S.A.',
    'En servicio (Habilitación Final)', 'FLORIDA', 'PARAJE PINTADO',
    '01/08/2014', '150', 50.00, -33.88554, -56.442846, 'Vestas', 'V112 - 3 MW', 3, 17);
  Parques.add(ap);

  ap := TParqueEolico.Create(1, 'NUEVO MANANTIAL CENTRAL 2',
    'NUEVO MANANTIAL S.A.', 'En servicio (Habilitación Final)', 'ROCHA',
    '19 DE ABRIL', '02/12/2009', '63', 4.00, -34.286155, -54.060287, '', '', 0, 0);
  Parques.add(ap);

  ap := TParqueEolico.Create(1, 'PARQUE EÓLICO LIBERTAD', 'TOGELY COMPANY S.A.',
    'En servicio (Habilitación Final)', 'SAN JOSÉ', 'SAN JOSÉ',
    '22/07/2014', '31.5', 7.70, -34.634590, -56.586462, 'VESTAS', 'V100-1.8MW', 1925, 4);
  Parques.add(ap);

  ap := TParqueEolico.Create(1, 'PARQUE EÓLICO LOMA ALTA - CENTRAL 1',
    'NUEVO MANANTIAL S.A.', 'En servicio (Habilitación Final)', 'ROCHA',
    '19 de Abril', '25/06/2008', '63', 7.80, -34.286155, -54.060287,
    'NedWind / Nordic Windpower', 'NW40 / 1000/59', 0, 25);
  Parques.add(ap);

  ap := TParqueEolico.Create(1, 'PARQUE EÓLICO MAGDALENA', 'KENTILUX S.A.',
    'En servicio (Habilitación Final)', 'SAN JOSÉ', '', '29/04/2011',
    '31.5', 17.20, -34.745844, -56.573067, 'Vestas', 'V90 2.0 MW / V100 1.8 MW', 0, 9);
  Parques.add(ap);

  ap := TParqueEolico.Create(1, 'PARQUE EÓLICO MALDONADO', 'R DEL SUR S.A.',
    'En servicio (Habilitación Final)', 'MALDONADO', 'Sierra de los Caracoles',
    '10/04/2014', '150', 50.00, -34.467151, -54.931297, 'Gamesa', 'Gamesa G97', 2, 25);
  Parques.add(ap);

  ap := TParqueEolico.Create(1, 'ENGRAW', 'ENGRAW EXPORT & IMPORT CO. S.A.',
    'En servicio (Habilitación Parcial)', 'FLORIDA', 'Fray Marcos',
    '25/10/2012', '31.5', 3.60, -34.204154, -55.74796, 'Vestas',
    'V100 - 1.8 MW', 1.8, 1);
  Parques.add(ap);

  ap := TParqueEolico.Create(1, 'LUZ DE LOMA', 'LUZ DE LOMA S.A.',
    'En servicio (Habilitación Parcial)', 'FLORIDA', 'PARAJE PINTADO',
    '27/08/2014', '31.5', 20.00, -33.866277, -56.340492, 'Vestas', 'V112 - 3 MW', 3, 7);
  Parques.add(ap);

  ap := TParqueEolico.Create(1, 'MINAS I', 'GENERACIÓN EÓLICA MINAS S.A. - GEMSA',
    'En servicio (Habilitación Parcial)', 'LAVALLEJA', '', '15/09/2014',
    '150', 42.00, -34.464674, -55.306549, 'Vestas', 'V112 - 3 MW', 3, 14);
  Parques.add(ap);

  ap := TParqueEolico.Create(1, 'PARQUE EÓLICO FLORIDA I', 'POLESINE S.A.',
    'En servicio (Habilitación Parcial)', 'FLORIDA', 'Ruta 56',
    '23/09/2014', '150', 50.00, -34.072142, -56.02797, 'Nordex', 'N117', 2.4, 21);
  Parques.add(ap);

  ap := TParqueEolico.Create(-1, 'ANDRESITO', 'UTE + Accionistas',
    'En suspenso', 'FLORES', 'Andresito', '', '', 50.00, -33.157526,
    -57.137823, '', '', 0, 0);
  Parques.add(ap);

  ap := TParqueEolico.Create(0, 'ARIAS', 'UTE + Accionistas',
    'Resolución de Adjudicación', 'FLORES', 'Colonia Arias', '', '',
    70.00, -33.828853, -56.573928, 'Gamesa', 'G114', 0, 0);
  Parques.add(ap);

  ap := TParqueEolico.Create(1, 'JUAN PABLO TERRA', 'UTE',
    'Resolución de Adjudicación', 'ARTIGAS', 'Javier de Viana', '',
    '150', 67.20, -30.408981, -56.640095, 'Nordex', 'N117', 2.4, 28);
  Parques.add(ap);

  ap := TParqueEolico.Create(0, 'PALOMAS', 'UTE', 'Resolución de Adjudicación',
    'SALTO', '', '', '', 71.00, -31.179298, -57.706112, 'Vestas', 'V112', 0, 0);
  Parques.add(ap);

  ap := TParqueEolico.Create(0, 'PAMPA', 'UTE + Accionistas',
    'Resolución de Adjudicación', 'TACUAREMBÓ', '', '', '150', 140.00,
    -32.246667, -56.215, 'Nordex', 'N117', 2.4, 0);
  Parques.add(ap);

  ap := TParqueEolico.Create(0, 'PARQUE EÓLICO ARTILLEROS', 'ROUAR S.A.',
    'Resolución de Adjudicación', 'COLONIA', 'Rosendo Mendoza',
    '31/1/2015', '150', 65.10, -34.349932, -57.555393, 'Suzlon', 'S 95', 2.1, 31);
  Parques.add(ap);

  ap := TParqueEolico.Create(0, 'VALENTINES', 'UTE + Accionistas',
    'Resolución de Adjudicación', 'FLORIDA', 'Valentines', '', '150',
    70.00, -33.267167, -55.10208, 'Gamesa', 'G114 T9m', 2, 35);
  Parques.add(ap);

  ap := TParqueEolico.Create(1, 'SANTA FE', 'LAVADERO DE LANAS BLENGIO S.A.',
    'Solicitud de Ampliación', 'SAN JOSÉ', 'Libertad', '24/07/2013',
    '31.5', 0.90, -34.647349, -56.593221, 'Vestas', 'V100', 1.8, 1);
  Parques.add(ap);

end;


procedure TForm1.Image1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: integer);
begin

end;

procedure TForm1.Image1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: integer);
begin
  StatusBar1.SimpleText := IntToStr(x) + ', ' + IntToStr(y);
end;

procedure TForm1.btBorrarBordeClick(Sender: TObject);

var
  r, g, b: integer;
  k, j: integer;
  cl: TColor;
  rgb: integer;
  intensidad: double;
  vk, vj: integer;
  wk, wj: integer;
  NColsTrianguloColores: integer;

  alfa: double;
  radio: integer;
  delta_k: double;

  ymin, ymax: array of integer;
  cnt_negros: array of integer;
  fsal: textfile;
  d: double;

  beta, alfa0, beta0: double;
  ralfa: double;
  rbeta: double;
  ralfa0: double;
  rbeta0: double;
  d_min: double;
  cnt_fracasos: integer;
  buscando: boolean;
  n_alfa, n_beta, n_alfa0, n_beta0: double;

  kp, y: integer;

begin

  wk := 4;
  wj := 4;


  // borro triángulo de colores
  NColsTrianguloColores := 100;
  for k := 0 to NColsTrianguloColores - 1 do
    for j := k to NColsTrianguloColores - 1 do
      c.Pixels[nCols - NColsTrianguloColores + j, k] := clWhite;


  // borro horizontales
  for j := 0 to nCols - 1 do
    for k := 0 to wk do
    begin
      c.Pixels[j, 0 + k] := clWhite;
      c.Pixels[j, nFilas - k] := clWhite;
    end;

  // borro verticales
  for k := 0 to nFilas do
    for j := 0 to wj do
    begin
      c.Pixels[0 + j, k] := clWhite;
      c.Pixels[nCols - j, k] := clWhite;
    end;



  setlength(ymin, nFilas);
  setlength(ymax, nFilas);
  setlength(cnt_negros, nFilas);

  for k := 0 to nFilas - 1 do
  begin
    cnt_negros[k] := 0;
    ymin[k] := nCols;
    ymax[k] := 0;
    for j := 0 to ncols do
    begin
      if c.Pixels[j, k] = clBlack then
      begin
        Inc(cnt_negros[k]);
        if j < ymin[k] then
          ymin[k] := j;
        if j > ymax[k] then
          ymax[k] := j;
        //       c.Pixels[ j, k ]:= clBlue;
      end;

    end;
  end;



  loadPatrones;

  alfa := 1;
  beta := 1;
  alfa0 := 0;
  beta0 := 0;
  ralfa := 1;
  rbeta := 1;
  ralfa0 := 300;
  rbeta0 := 300;


  d_min := distancia(ymin, ymax, cnt_negros, alfa, beta, alfa0, beta0);

  cnt_fracasos := 0;
  buscando := True;


  (**
  while buscando do
  begin
    n_alfa:= alfa +    ralfa * ( 1 - 2* random);
    n_alfa:= max( n_alfa, 0.5);
    n_alfa:= min( n_alfa, 2 );

    n_beta:= beta +    rbeta * ( 1 - 2* random);
    n_beta:= max( n_beta, 0.5);
    n_beta:= min( n_beta, 2 );

    n_alfa0:= alfa0 +    ralfa0 * ( 1 - 2* random);
    n_alfa0:= max( n_alfa0, -300);
    n_alfa0:= min( n_alfa0, 300 );

    n_beta0:= beta0 +    rbeta0 * ( 1 - 2* random);
    n_beta0:= max( n_beta0, -300);
    n_beta0:= min( n_beta0, 300 );

    n_beta:= n_alfa;
    d:= distancia( ymin, ymax, cnt_negros, n_alfa, n_beta, n_alfa0, n_beta0);

    if d < d_min then
    begin
      cnt_fracasos:= 0;
      alfa:= n_alfa;
      beta:= n_beta;
      alfa0:= n_alfa0;
      beta0:= n_beta0;
      d_min:= d;
      writeln( 'd: ', d );
    end
    else
    begin
      inc( cnt_fracasos );
      if cnt_fracasos > 10000 then
      begin
        cnt_fracasos:= 0;
        ralfa:= ralfa * 0.95;
        rbeta:= rbeta * 0.95;
        ralfa0:= ralfa0 * 0.95;
        rbeta0:= rbeta0* 0.95;
        if ralfa < 1e-8 then
          buscando:= false;
      end;
    end;
  end;

  writeln( 'alfa: ',alfa );
  writeln( 'beta: ', beta );
  writeln( 'alfa0:', alfa0 );
  writeln( 'beta0: ', beta0 );
  writeln( 'd: ', d );


  for k:= 0 to 500-1 do
  begin
    kp:=  trunc( k * alfa + alfa0  + 0.5);
    if (kp > 0) and (kp < 500 ) then
    begin
      y:= trunc( (patron_ymin[kp]- beta0 )/ beta + 0.5);
      if (y > 0 ) and ( y < nCols ) then
        c.Pixels[y, k]:= clGreen;

      y:= trunc( (patron_ymax[kp]- beta0 )/ beta + 0.5);
      if (y > 0 ) and ( y < nCols ) then
        c.Pixels[y, k]:= clRed;
    end;
  end;


     *)


(*
  assignfile( fsal, 'yminmaxcnt.xlt' );
  rewrite( fsal );
  writeln( fsal, 'fila'#9'ymin'#9'ymax'#9'cnt');
  for k:= 0 to nFilas-1 do
    writeln( fsal, k, #9, ymin[k], #9, ymax[k], #9, cnt_negros[k] );
  closefile( fsal );
  *)
end;

procedure TForm1.btGenGPFClick(Sender: TObject);
var
  idx: TDatosIdx;
begin
  idx:= ScanIdx;


end;



function v2p(v_mps: double): double;
var
  k: integer;
begin
  if v_mps <= 3.3 then
  begin
    Result := 0;
    exit;
  end;
  if v_mps >= 15 then
  begin
    Result := 1;
    exit;
  end;
  Result := 1.0 / (1.0 + EXP(-0.594709117 * (v_mps - 8.888780408)));
end;


procedure TForm1.btPlotParquesClick(Sender: TObject);
var
  k: integer;
  ap: TParqueEolico;
  PotenciaInstalada, PotenciaPrevista: double;
  Vmedia, pMW: double;
  fsal: textfile;
  khora: integer;

begin
  assignfile(fsal, 'peolicos_'+ eBaseArchi.Text+eRodada.text+'.xlt');
  rewrite(fsal);


  for k := 0 to Parques.Count - 1 do
  begin
    ap := Parques[k];
    if ap.activo = 1 then
      Write(fsal, #9, ap.Potencia_Autorizada_MW: 12: 2);
  end;
  writeln(fsal);

  for k := 0 to Parques.Count - 1 do
  begin
    ap := Parques[k];
    if ap.activo = 1 then
      Write(fsal, #9, ap.CentralGeneradora);
  end;
  writeln(fsal);

  for k := 0 to Parques.Count - 1 do
  begin
    ap := Parques[k];
    if ap.activo = 1 then
      Write(fsal, #9, ap.AgenteGenerador);
  end;
  writeln(fsal);


  for khora := 1 to 13 do
  begin

    Image1.Picture.LoadFromFile('c:\basura\foto'+eBaseArchi.Text+eRodada.text + '_'+pad(khora) + '.png');

    inicializar;
    PotenciaInstalada := 0;
    PotenciaPrevista := 0;

    Write(fsal, kHora);

    for k := 0 to Parques.Count - 1 do
    begin
      ap := Parques[k];
      if ap.Activo = 1 then
      begin
        PotenciaInstalada := PotenciaInstalada + ap.Potencia_Autorizada_MW;
        Vmedia := CalcVMedia(ap.top, ap.left) / 3.6 * FactorDePerdida_V;
        pMW := ap.Potencia_Autorizada_MW * v2p(VMedia);
        PotenciaPrevista := PotenciaPrevista + pMW;
        Write(fsal, #9, pMW: 12: 2);
      end;
    end;
    writeln(fsal);
    application.ProcessMessages;
    sleep(1);
  end;

  closefile(fsal);

  memo1.Lines.add('Potencia instalada [MW]: ' + FloatToStr(PotenciaInstalada));
  memo1.Lines.add('Potencia prevista [MW]: ' + FLoatToStr(PotenciaPrevista));
  if PotenciaInstalada > 0 then
    memo1.Lines.add('FC[%]: ' + FLoatToStr(
      trunc(PotenciaPrevista / PotenciaInstalada * 1000) / 10));

  for k := 0 to Parques.Count - 1 do
  begin
    ap := Parques[k];

    case ap.Activo of
      0: // en construcción
      begin
        c.Brush.Color := clYellow;
        c.EllipseC(ap.left, ap.top, 3, 3);
      end;
      1: // en operación
      begin
        c.Brush.Color := clGreen;
        c.EllipseC(ap.left, ap.top, 3, 3);
      end;

      else // dudosos
      begin
        c.Brush.Color := clBlack;
        c.EllipseC(ap.left, ap.top, 3, 3);
      end;
    end;
  end;
end;

procedure TForm1.btSGE_UTEClick(Sender: TObject);
begin

  //  'bal'  'hid' 'ter' 'eol' 'bio' 'fot'

  ScanDatosPorDia( 'bal' );
  ScanDatosPorDia( 'hid' );
  ScanDatosPorDia( 'ter' );
  ScanDatosPorDia( 'eol' );
  ScanDatosPorDia( 'bio' );
  ScanDatosPorDia( 'fot' );


end;

end.
