unit uFormularioEstatico;

{$mode delphi}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, EditBtn, Graphics, StdCtrls,
  ExtCtrls, maskedit, PopupNotifier, uFormularioBase, ucosa, ufechas, contnrs,
  uFormularioAutomatico, uvisordetabla;

type

  TIniProcedures = procedure of object;
  { TFormularioEstatico }

  TFormularioEstatico = class(TFormularioBase)

    PopupNotifier: TPopupNotifier;

    procedure TButton_OnClick(Sender: TObject);

    procedure FormCreate(Sender: TObject);
    procedure TCheckBox_OnChange(Sender: TObject);
    procedure TRadioGroup_OnClick(Sender: TObject);
    procedure OnExit(Sender: TObject);

  public
    { public declarations }


    IgnoranceList: TFPStringHashTable;

    // Es una hash donde la clave es el nombre del control
    // y el puntero es el procedimiento que lo carga.
//    SpecificControls: TFPHashList;

    constructor Create(TheOwner: TComponent); override; overload;

    function AsignarEventosYCargarControlesEstandar: boolean; virtual;
    procedure CargarControlesEspecificos;

    procedure SetEvents(); virtual;
    procedure RunEvents(); virtual;

  end;

var
  FormularioEstatico: TFormularioEstatico;

implementation

uses
  uRegistroDeEditores
  ;

{$R *.lfm}

{ TFormularioEstatico }

function TFormularioEstatico.AsignarEventosYCargarControlesEstandar: boolean;
var
  i, j, tag: integer;
  dt: TDataTableOfCosa;
  DataColumn: TDataColumnOfCosa;
  DataCell: TDataCellOfCosa;
  c: TComponent;

  bval: boolean;
  fecha: TFecha;


  IgnoranceList: TStringList;
  b: String;
  a: Integer;
  aTop: Integer;
  aLeft: Integer;
  dta: TDataTableOfCosa;
  Grid: TEditorDataGridControl;
  LocalDataColumn: TDataColumnOfCosa;
  l: Integer;
  p: ^TIniProcedures;
  dc: String;
begin
  //if (IgnoreTable='') and (IgnoreList = '') then
  //  exit;

  Result:=true;

  self.SetEvents();

  for i := 0 to dataSet.DataTableListCount - 1 do
  begin
    dt := dataSet.DataTableList[i];

    if dt.TableName='dt_TListaEscenarios' then
      a:=1;

    IgnoranceList:= TStringList.Create;
    IgnoranceList.CommaText:=self.IgnoranceList.Items[dt.TableName];
    IgnoranceList.CaseSensitive := False;

    for j := 0 to dt.DataColumnListCount - 1 do
    begin

      DataColumn:=dt.DataColumnList[j];

      //se asume que las tablas tienen un solo datarow
      DataCell:=dt.DataRowList[0].DataCellList[j];

      //PODRÍAMOS AGREGAR UN CONTROL BOLUDO QUE CONTROLE QUE LOS INDEX SEAN IGUALES...

      {dfusco@20150512
       Si el DATASET tiene más de una tabla busca controles con el formato de
       nombre: [DATA_TABLE_NAME]_[DATA_COLUMN_NAME]. Sino, busca sólo por el
       nombre de la columna.}
      if DataSet.DataTableListCount>1 then
      begin
        l:=self.LocalDataColumnList.AddDataColumn(DataColumn.Clone(),false);
        self.LocalDataColumnList.Items[l].ColumnName:=Copy(dt.TableName,4,Length(dt.TableName))+'_'+DataColumn.ColumnName;
      end
      else
        l:=self.LocalDataColumnList.AddDataColumn(DataColumn.Clone(),false);

      self.LocalDataColumnList.Items[l].DataColumnListOwner:=DataColumn.DataColumnListOwner;

      tag:=self.LocalDataRow.AddDataCell(DataCell, false);

      // Se verifica si el control tiene una forma particular de cargarse
      // p:=SpecificControls.Find(dt.TableName+'.'+DataColumn.ColumnName);

      if (IgnoranceList.CommaText<>'*') and
        (IgnoranceList.IndexOf(DataColumn.ColumnName)<0) then
      begin

        c := self.FindComponent(self.LocalDataColumnList.Items[tag].ColumnName);

        {$IFDEF DEBUG}
        if not Assigned(c) then
          raise Exception.Create('No existe control para el campo ' + self.LocalDataColumnList.Items[tag].ColumnName);
        {$ENDIF}

        c.Tag := tag;

        //**********************************************************************
        //* dfusco@20150413
        //*
        //* IMPORTANTE
        //* ----------
        //*
        //* Si se necesita "sobreescribir" el evento OnExit en un formulario,
        //* es necesario llamar previamente al método UpdateCell que, además de
        //* actualizar la celda de la base de datos, hace el chequeo de errores.
        //*
        //* EJEMPLO:
        //*
        //* procedure TFormEditSimSEE.TGlobs_NPostesExit(Sender: TObject);
        //* begin
        //*   if self.UpdateDataCell(Sender) then
        //*     self.TGlobs_durPos.ColCount:=StrToInt(self.TGlobs_NPostes.Text)+1;
        //* end;
        //*
        //**********************************************************************
        if not assigned(TWinControl(c).OnExit) then
          TWinControl(c).OnExit := OnExit();

        dta:= DataCell.DataRowOwner.DataTableOwner;

        if (DataColumn.DataType is TDataTypeBooleanOfCosa) then
        begin

          {$IFDEF DEBUG}
          if (not (c is TCheckBox)) and (not (c is TRadioGroup) and (not (c is TRadioButton))) then
            raise Exception.Create(DataColumn.ColumnName + ' VER');
          {$ENDIF}

          DataColumn.DataType.Eval(DataCell.sval, @bval);

          if c is TCheckBox then
          begin

            //ESTO SE HACE PARA QUE SE DISPARE EL EVENTO ASIGNADO:
            //TCheckBox(c).Checked := not bval;
            TCheckBox(c).Checked := bval;
          end

          else if c is TRadioGroup then
          begin

            if bval then
              TRadioGroup(c).ItemIndex:=1
            else
              TRadioGroup(c).ItemIndex:=0;
          end

          else if c is TRadioButton then
          begin

            //ESTO SE HACE PARA QUE SE DISPARE EL EVENTO ASIGNADO:
            //TRadioButton(c).Checked := not bval;
            TRadioButton(c).Checked := bval;
          end;

        end

        else if (DataColumn.DataType is TDataTypeDateOfCosa) then
        begin
          {$IFDEF DEBUG}
          if not (c is TDateEdit) then
            raise Exception.Create(DataColumn.ColumnName + ' VER');
          {$ENDIF}

          TDateEdit(c).Text := DataCell.sval;
        end

        else if (DataColumn.DataType is TDataTypeIntegerOfCosa) or
                (DataColumn.DataType is TDataTypeStringOfCosa)  or
                (DataColumn.DataType is TDataTypeDoubleOfCosa)  then
        begin
          {$IFDEF DEBUG}
          if (not (c is TEdit)) and (not (c is TRadioGroup)) then
            raise Exception.Create(DataColumn.ColumnName + ' VER');
          {$ENDIF}

          if c is TEdit then
            TEdit(c).Text := DataCell.sval;

          if c is TRadioGroup then
            TRadioGroup(c).ItemIndex:= StrToInt(DataCell.sval);


        end

        else if (DataColumn.DataType is TDataTypeListOfCosa) then
        begin
          {$IFDEF DEBUG}
          if not (c is TPanel) then
            raise Exception.Create(DataColumn.ColumnName + ' VER');
          {$ENDIF}

          //Grid:=TEditorDataGridControl.Create(dt.TableName+'_'+DataColumn.ColumnName,
          //self, TPanel(c), tag, dt.GetDataColumnList().ItemListClaseDeCosa);
          //
          //Grid.AVT_DefaultEditor.Top:=0;
          //Grid.AVT_DefaultEditor.Left:=0;
          //
          //Grid.Place(0,0);
          //
          //Grid.OnExit:=OnExit();
        end

        else if DataColumn.DataType is TDataTypeArchiRefOfCosa then
        begin
          TEdit(c).Text := DataCell.sval;
        end

        else if DataColumn.DataType is TDataTypeDataRowOfCosa then
        begin
          {$IFDEF DEBUG}
          if not (c is TButton) then
            raise Exception.Create(DataColumn.ColumnName + ' VER');
          {$ENDIF}

          TButton(c).OnClick := self.TButton_OnClick;
        end;

      end;
    end;
  end;

  self.RunEvents();

end;

procedure TFormularioEstatico.CargarControlesEspecificos;
var
  p: TProcedure;
  i: Integer;
begin
  //for i:=0 to SpecificControls.Count do
  //begin
  //  p:= TProcedure( SpecificControls.Items[i] );
  //  p;
  //end;
end;

procedure TFormularioEstatico.SetEvents;
begin

end;

procedure TFormularioEstatico.RunEvents;
begin

end;

procedure TFormularioEstatico.TCheckBox_OnChange(Sender: TObject);
var
  DataCell: TDataCellOfCosa;
begin
  self.hayCambios := True;

  DataCell:= self.LocalDataRow.DataCellList[TWinControl(Sender).Tag];
  DataCell.sval:=BoolToStr(TCheckBox(Sender).Checked);
end;

procedure TFormularioEstatico.FormCreate(Sender: TObject);
begin
  inherited;
end;

procedure TFormularioEstatico.TButton_OnClick(Sender: TObject);
var
  DataColumn: TDataColumnOfCosa;
  DataCell: TDataCellOfCosa;
  ADataSet: TDataStore;
  split: TStringList;
  DataTableName: String;
  DataRowId: Integer;
  CosaStrId: String;
  frmChild: TFormularioBase;
begin

  DataColumn:=self.LocalDataTable.DataColumnList[TWinControl(Sender).Tag];
  DataCell:=self.LocalDataRow.DataCellList[TWinControl(Sender).Tag];

  split:=TStringList.Create();
  ExtractStrings(['.'], [], PChar(DataCell.sval), split);
  DataTableName:= split[0];
  DataRowId:= StrToInt(split[1]);

  ADataSet:=TDataStore.Create(self.DataSet);
  self.DataSet.Query('*', DataTableName, DataRowId, ADataSet);

  //ucosa.PrintDataSet(ADataSet);

  CosaStrId:=copy(DataTableName, 4, Length(DataTableName));

  frmChild:=uRegistroDeEditores.OpenForm(self,
                                         CosaStrId,
                                         ADataSet,
                                         self.Caption+' >> '+ 'PEPE');

  if frmChild.ShowModal() = mrOk then
    self.DataSet.Merge(frmChild.DataSet)

end;

procedure TFormularioEstatico.TRadioGroup_OnClick(Sender: TObject);
var
  DataCell: TDataCellOfCosa;
begin
  self.hayCambios := True;

  DataCell:= self.LocalDataRow.DataCellList[TWinControl(Sender).Tag];
  DataCell.sval:=IntToStr(TRadioGroup(Sender).ItemIndex);
end;

procedure TFormularioEstatico.OnExit(Sender: TObject);
begin
  self.UpdateDataCell(Sender);
end;

constructor TFormularioEstatico.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);

  self.IgnoranceList:=TFPStringHashTable.Create();
  //self.SpecificControls:= TFPHashList.Create;
end;

end.

