unit uescenarios;
{
rch@20130914
Un escenario está definido por un conjunto de capas activas.
Las capas son números enteros
}


interface

uses
  Classes, SysUtils, xMatDefs, uCosa, uCosaConNombre;

type

  { TEscenario_rec }

  TEscenario_rec = class( TCosaConNombre )
  public
    capasActivas: TDAOfNInt;
    activa:       boolean;
    descripcion:  string;
    run_opt:      boolean;
    run_sim:      boolean;

    constructor Create( nombre: string );

    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto ); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}

    procedure Free; override;
    procedure AddCapa( capa: integer );
    procedure DelCapa( capa: integer );
    function tieneCapa( capa: integer): boolean;

    // Chequea si la Cosa está en una capa de las activas
    // del escenario y retonra TRUE en caso afirmativo.
    function Participa( Cosa: TCosa ): boolean;

 {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer = -2): TDataColumnListOfCosa; override;
 {$ENDIF}

  published
    {$IFDEF MIGRACION_PERSISTENCIA}
    property _capasActivas: TDAOfNInt read capasActivas write capasActivas;
    property _activa:       boolean   read activa       write activa;
    property _descripcion:  string    read descripcion  write descripcion;
    property _run_opt:      boolean   read run_opt      write run_opt;
    property _run_sim:      boolean   read run_sim      write run_sim;
    {$ENDIF}

  end;

  { TListaEscenarios }

  TListaEscenarios = class(TListaDeCosasConNombre)
    constructor Create(nombre: string);

    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText( f: TArchiTexto); override;
    {$ENDIF}

// chequea si el escenario ya no está en la lista y lo agrega.
// retorna el puntero al Rec agregado o NIL si ya estaba y no fue agregado.
    function AppendEscenario( nombreEscenario: string ): TEscenario_rec;

    function lista_activas: TStrings;
    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer = -2): TDataColumnListOfCosa; override;
    {$ENDIF}

end;

procedure AlInicio;
procedure AlFinal;

implementation


constructor TEscenario_rec.Create( nombre: string );
begin
  inherited Create(0, nombre );
  setlength( capasActivas, 1 );
  capasActivas[0]:= 0;
  activa:= true;
  run_opt:= true;
  run_sim:= true;
end;



{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TEscenario_rec.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText( f );
  f.rd( 'capasActivas', capasActivas );
  f.rd( 'activa', activa );
  f.rd( 'descripcion', descripcion );
  f.rd( 'run_opt', run_opt );
  f.rd( 'run_sim', run_sim );
end;
{$ENDIF}

{$IFNDEF MIGRACION_PERSISTENCIA}
procedure TEscenario_rec.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText( f );
  f.wr( 'capasActivas', capasActivas );
  f.wr( 'activa', activa );
  f.wr( 'descripcion', descripcion );
  f.wr( 'run_opt', run_opt );
  f.wr( 'run_sim', run_sim );
end;
{$ENDIF}

procedure TEscenario_rec.Free;
begin
  setlength( capasActivas, 0 );
  inherited Free;
end;

procedure TEscenario_rec.AddCapa( capa: integer );
begin
  addToArray( capasActivas, capa );
end;

procedure TEscenario_rec.DelCapa( capa: integer );
var
  arr: TDAofNInt;
begin
  arr:=capasActivas;
  delFromArray( arr, capa );
  capasActivas:=arr;

end;

function TEscenario_rec.tieneCapa( capa: integer): boolean;
begin
  result:= kInArray( capasActivas, capa ) >= 0;
end;


function TEscenario_rec.Participa( Cosa: TCosa ): boolean;
begin
  result:= kInArray( capasActivas, Cosa.capa ) >= 0;
end;


{$IFDEF MIGRACION_PERSISTENCIA}
class function TEscenario_rec.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa, xVersion);
  Result.AddDAOfIntegerColumn('capasActivas' );
  Result.AddBooleanColumn    ('activa',      true);
  Result.AddStringColumn     ('descripcion', ''  );
  Result.AddBooleanColumn    ('run_opt',     true);
  Result.AddBooleanColumn    ('run_sim',     true);
end;
{$ENDIF}

constructor TListaEscenarios.Create(nombre: string);
var
 ae: TEscenario_rec;
begin
  inherited Create(0, nombre );
  // por defecto ponemos escenario base con capa cero
  ae:= TEscenario_rec.Create('Base');
  add( ae );
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TListaEscenarios.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText( f );
end;
{$ENDIF}

{$IFNDEF MIGRACION_PERSISTENCIA}
procedure TListaEscenarios.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText( f );
end;
{$ENDIF}




function TListaEscenarios.lista_activas: TStrings;

var
 k: integer;
 rec: TEscenario_rec;
 res: TStringList;

begin
  res:= TStringList.Create;
  for k:= 0 to count -1 do
  begin
    rec:= items[k] as TEscenario_rec;
    if rec.activa then
       res.Add( rec.nombre );
  end;
  result:= res;
end;

{$IFDEF MIGRACION_PERSISTENCIA}
class function TListaEscenarios.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa, xVersion);
  Result.ItemListClaseDeCosa:=TEscenario_rec;
end;
{$ENDIF}

function TListaEscenarios.AppendEscenario( nombreEscenario: string ): TEscenario_rec;
var
 rec: TEscenario_rec;
begin
  if Find( nombreEscenario ) = nil then
  begin
    rec:= TEscenario_rec.Create( nombreEscenario );
    add( rec );
    result:= rec;
  end
  else
    result:= nil;
end;



(*********************************)
procedure AlInicio;
begin
  registrarClaseDeCosa(TEscenario_rec.ClassName, TEscenario_rec );
  registrarClaseDeCosa(TListaEscenarios.ClassName, TListaEscenarios );
end;

procedure AlFinal;
begin
end;

end.

